// @(#)root/net:$Id: TWebFile.h 30365 2009-09-23 09:02:58Z rdm $
// Author: Fons Rademakers   17/01/97

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_TWebFile
#define ROOT_TWebFile


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TWebFile                                                             //
//                                                                      //
// A TWebFile is like a normal TFile except that it reads its data      //
// via a standard apache web server. A TWebFile is a read-only file.    //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TFile
#include "TFile.h"
#endif
#ifndef ROOT_TUrl
#include "TUrl.h"
#endif
#ifndef ROOT_TSystem
#include "TSystem.h"
#endif

class TSocket;
class TWebSocket;


class TWebFile : public TFile {

friend class TWebSocket;
friend class TWebSystem;

private:
   mutable Long64_t  fSize;             // file size
   TSocket          *fSocket;           // socket for HTTP/1.1 (stays alive between calls)
   TUrl              fProxy;            // proxy URL
   Bool_t            fHasModRoot;       // true if server has mod_root installed
   Bool_t            fHTTP11;           // true if server support HTTP/1.1
   Bool_t            fNoProxy;          // don't use proxy
   TString           fMsgReadBuffer;    // cache ReadBuffer() msg
   TString           fMsgReadBuffer10;  // cache ReadBuffer10() msg
   TString           fMsgReadBuffers;   // cache ReadBuffers() msg
   TString           fMsgReadBuffers10; // cache ReadBuffers10() msg
   TString           fMsgGetHead;       // cache GetHead() msg

   static TUrl       fgProxy;           // globally set proxy URL

   TWebFile() : fSocket(0) { }
   void        Init(Bool_t readHeadOnly);
   void        CheckProxy();
   TString     BasicAuthentication();
   Int_t       GetHead();
   Int_t       GetLine(TSocket *s, char *line, Int_t maxsize);
   Int_t       GetHunk(TSocket *s, char *hunk, Int_t maxsize);
   const char *HttpTerminator(const char *start, const char *peeked, Int_t peeklen);
   Int_t       GetFromWeb(char *buf, Int_t len, const TString &msg);
   Int_t       GetFromWeb10(char *buf, Int_t len, const TString &msg);
   Bool_t      ReadBuffer10(char *buf, Int_t len);
   Bool_t      ReadBuffers10(char *buf, Long64_t *pos, Int_t *len, Int_t nbuf);

public:
   TWebFile(const char *url, Option_t *opt="");
   TWebFile(TUrl url, Option_t *opt="");
   virtual ~TWebFile();

   Long64_t    GetSize() const;
   Bool_t      IsOpen() const;
   Int_t       ReOpen(Option_t *mode);
   Bool_t      ReadBuffer(char *buf, Int_t len);
   Bool_t      ReadBuffers(char *buf, Long64_t *pos, Int_t *len, Int_t nbuf);
   void        Seek(Long64_t offset, ERelativeTo pos = kBeg);

   static void        SetProxy(const char *url);
   static const char *GetProxy();

   ClassDef(TWebFile,1)  //A ROOT file that reads via a http server
};


class TWebSystem : public TSystem {

private:
   void *fDirp;    // directory handler

   void *GetDirPtr() const { return fDirp; }

public:
   TWebSystem();
   virtual ~TWebSystem() { }

   Int_t       MakeDirectory(const char *name);
   void       *OpenDirectory(const char *name);
   void        FreeDirectory(void *dirp);
   const char *GetDirEntry(void *dirp);
   Int_t       GetPathInfo(const char *path, FileStat_t &buf);
   Bool_t      AccessPathName(const char *path, EAccessMode mode);
   Int_t       Unlink(const char *path);

   ClassDef(TWebSystem,0)  // Directory handler for HTTP (TWebFiles)
};

#endif
